#define	VERSION	"1.0.4"
/************************************************************************/
/*									*/
/*	Title		:	RockForce User Mode SideBand Test Util	*/
/*									*/
/*	Author		:	N.P.Vassallo				*/
/*									*/
/*	Creation	:	28th February 2004			*/
/*									*/
/*	Description	:	RockForce SideBand Functions		*/
/*									*/
/************************************************************************/

/* History...

Version	Date		Author	Comments
=======	====		======	========
1.0.0	23-Dec-04	NPV	Creation.
1.0.1	05-Feb-07	Lee	fix to work per docs, /proc/ioports support
1.0.2	07-Feb-07	Lee	move all SysTimeDelay into _uinb and _uoutb
1.0.3	20-Mar-07	Lee	fix /proc/ioports parsing to look for the card, not the bus
1.0.4	27-Oct-07	Lee	fix /proc/ioports parsing to look for the card, not the port

*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<unistd.h>
#include        <sys/types.h>
#include        <sys/timeb.h>
#include	<sys/file.h>
#include	<sys/ioctl.h>

#include	"sysdefns.h"
#include	"rfsbapi.h"

/*****************************************************************************
*******************************                *******************************
*******************************   Prototypes   *******************************
*******************************                *******************************
*****************************************************************************/

#define	ReadReg(A)		_uinb(hDevice+A)
#define	WriteReg(A,B)		_uoutb(hDevice+A,B)
#define	ReadUARTReg(A)		ReadUARTRegister(hDevice,Port,A)
#define	WriteUARTReg(A,B)	WriteUARTRegister(hDevice,Port,A,B)

_u8	Read1WireBus(_u32 hDevice);
void	Write1WireBus(_u32 hDevice,_u8 Value);
_u8	Fcs8(_u8 Crc8,_u8 x);

_u8	_uinb(_u32 port);
void	_uoutb(_u32 port,_u8 val);

_u8	ReadUARTRegister(_u32 hDevice,_u32 Port,_u32 Reg);
void	WriteUARTRegister(_u32 hDevice,_u32 Port,_u32 Reg,_u8 Val);

_u32	SysTimeStamp(void);
_u32	SysTimeElapsed(_u32 TimeStampmS);
_u32	SysTimeDelay(_u32 DelaymS);

int ctohex(char* c)
{
	if (*c >= 0x30 && *c <= 0x39) return (*c - 0x30);
	if (*c >= 0x61 && *c <= 0x66) return (*c - 0x61 + 10); 
	return (-1);
}

/*****************************************************************************
**********************************          **********************************
**********************************   main   **********************************
**********************************          **********************************
*****************************************************************************/

int	hIO = 0;		/* Handle for I/O port access */

#define	BUF_SIZE	1024

int	main(int argc,char **argv)
{
	_u32	UartIO = 0;
	_u32	UartLatch = 0;
	_u32	SBDevice;
	_u32	SBIndex;
	char	*ptr = "";
	char	*pCommand = NULL;
	char	*pDevName = NULL;
	char	ErrBuf[100];

	printf("RFUSER v%s RockForce SideBand Test Utility.\n",VERSION);
	printf("Copyright (c) 2004, 2007 Mainpine Limited\n\n");
	if((argc==1)||((--argc>=1)&&((strcmp(*(argv+1),"/h")==0)||(strcmp(*(argv+1),"/?")==0))))
	{
		printf(argv[0]);
		printf(" [ /h | /? | <uart=x,y>|<device> [ serial ] [ reset | dumpuart ] ]\n\n");
		printf("Where:\n");
		printf("/h or /?           generates this help screen\n");
		printf("<uart=x,y>         specifies the device to access,\n");
		printf("                   x=uart address (e.g. 0xE000)\n");
		printf("                   y=latch address (e.g. 0xE060)\n");
		printf("serial             obtain hardware serial number\n");
		printf("reset              perform hardware reset\n");
		printf("dumpuart           display contents of UART registers\n");
		printf("\n");
		exit(0);
	}

	/*
	** Access I/O space...
	*/

	if((hIO = open("/dev/port",O_RDWR)) < 0) 
	{
		fprintf(stderr,"Cannot open /dev/port\n\n");
		exit(1);
	}

	/*
	** Process command line... 
	*/

	while(argc--)
	{
		ptr = *++argv;			/* Get the argument */

		if(strcmp(ptr,"serial") == 0)
		{
			pCommand = ptr;
			continue;
		}

		if(strncmp(ptr,"uart=",5) == 0)
		{
			char	*ptrnext = NULL;

			UartIO = strtoul(ptr+5,&ptrnext,0);
			if((ptrnext)&&(*ptrnext == ','))
			{
				UartLatch = strtoul(ptrnext+1,NULL,0);
			}

			if((UartIO % 8)||(UartLatch % 8))
			{
				fprintf(stderr,"ERROR: UART(0x%04X,0x%04X) not 8 byte aligned.\n\n", (u_int) UartIO, (u_int) UartLatch);
				close(hIO);
				exit(1);
			}

			SBDevice = UartLatch - 0x60;
			SBIndex = (UartIO - SBDevice) / 8;

			if(SBIndex > 7)
			{
				fprintf(stderr,"ERROR: UART(0x%04X,0x%04X) invalid addresses.\n\n", (u_int) UartIO, (u_int) UartLatch);
				close(hIO);
				exit(1);
			}
			continue;
		}

		if(strcmp(ptr,"reset") == 0)
		{
			pCommand = ptr;
			continue;
		}

		if(strcmp(ptr,"wedge") == 0)
		{
			pCommand = ptr;
			continue;
		}

		if(strcmp(ptr,"dumpuart") == 0)
		{
			pCommand = ptr;
			continue;
		}

		pDevName = ptr;
	}

	/*
	** Get device resources...
	*/

	if(pDevName)
	{
		struct serial_struct serinfo;
		struct serial_multiport_struct multi;
		int	fd;

		if ((fd = open(pDevName, O_RDONLY|O_NONBLOCK)) < 0)
		{
			perror(pDevName);
			exit(1);
		}

		serinfo.reserved_char[0] = 0;
		if (ioctl(fd, TIOCGSERIAL, &serinfo) < 0) 
		{
			perror("Cannot get serial info");
			close(fd);
			exit(1);
		}
		UartIO = serinfo.port;

		if (ioctl(fd, TIOCSERGETMULTI, &multi) < 0)
		{
			/* Some kernels may not have serial multiport capability enabled.
			   So to address the latch I/O we need to look for the base I/O
			   for the PCI card in /proc/ioports and then add 60h. */

			multi.port1 = -1;
			int ioportsfd = open("/proc/ioports", O_RDONLY);
			if (ioportsfd < 0)
			{
				perror("Cannot open /proc/ioports");
				close(fd);
				exit(1);
			}
			int ioportscc, ioportspos = 0, spaces;
			char* ioportsline = malloc(1024);
			memset(ioportsline, 0, 1024);
			while ((ioportscc = read(ioportsfd, &ioportsline[ioportspos], 1)))
			{
				if (ioportsline[ioportspos] == '\n' || ioportspos == 1023)
				{
					ioportsline[ioportspos] = '\0';
					spaces = 0;
					/*
					 * We're searching for a card, not a bus, and not a serial port.
					 * A card is identified by the hardware ID of a form: XXXX:XX:XX.X
					 * where the "X" is a hexadecimal number.  Ignore leading whitespace.
					 */
					while (*ioportsline == 0x20)
					{
						ioportsline++;
						spaces++;
					}

					if ((ioportsline[0] >= 0x30 && ioportsline[0] <= 0x39) || (ioportsline[0] >= 0x61 && ioportsline[0] <= 0x66))
					{
						char* delimpos = strchr(ioportsline, '-');
						char* endpos = strchr(ioportsline, ':');

						/*
						 * Determine if this is a card by checking for the form.
						 */
						int iscard = 0;
						if (ctohex(endpos+2) >= 0 && ctohex(endpos+3) >= 0 &&
						    ctohex(endpos+4) >= 0 && ctohex(endpos+5) >= 0 &&
						    *(endpos+6) == ':' &&
						    ctohex(endpos+7) >= 0 && ctohex(endpos+8) >= 0 &&
						    *(endpos+9) == ':' &&
						    ctohex(endpos+10) >= 0 && ctohex(endpos+11) >= 0) iscard = 1;

						if (iscard && delimpos && endpos)
						{
							while (*(endpos-1) == ' ') endpos--;
							int startio = 0, endio = 0;
							char* parsepos;
							for (parsepos = ioportsline; parsepos < delimpos; parsepos++)
							{
								startio <<= 4;
								startio += ctohex(parsepos);
							}
							for (parsepos = delimpos+1; parsepos < endpos; parsepos++)
							{
								endio <<= 4;
								endio += ctohex(parsepos);
							}
							if (UartIO >= startio && UartIO <= endio)
							{
								multi.port1 = startio + 0x60;
							}
						}
					}
					ioportsline -= spaces;
					memset(ioportsline, 0, 1024);
					ioportspos = 0;
				}
				else ioportspos++;
			}
			close(ioportsfd);
			if (multi.port1 == -1)
			{
				fprintf(stderr,"ERROR: Unable to determine serial multiport (latch) info.\n\n");
				close(hIO);
				exit(1);
			}
		}
		UartLatch = multi.port1;

		if((UartIO % 8)||(UartLatch % 8))
		{
			fprintf(stderr,"ERROR: UART(0x%04X,0x%04X) not 8 byte aligned.\n\n", (u_int) UartIO, (u_int) UartLatch);
			close(hIO);
			exit(1);
		}

		SBDevice = UartLatch - 0x60;
		SBIndex = (UartIO - SBDevice) / 8;

		if(SBIndex > 7)
		{
			fprintf(stderr,"ERROR: UART(0x%04X,0x%04X) invalid addresses.\n\n", (u_int) UartIO, (u_int) UartLatch);
			close(hIO);
			exit(1);
		}

		printf("%s port  = 0x%04X\n",pDevName,UartIO);
		printf("%s latch = 0x%04X\n",pDevName,UartLatch);
	}

	/*
	** Read board serial number...
	*/

	{
		_u8	SerialBuffer[40];
		_u32	SerialReturnLen = 0;
		_u32	Status;
		_u32	loop;

		Status = RockForceReadSerialNumber(SBDevice,SerialBuffer,sizeof(SerialBuffer),&SerialReturnLen);

		if(Status == RF_SUCCESS)
		{
			if(pCommand && strcmp(pCommand,"serial") == 0)
			{
				printf("Serial Number = ");

				if(SerialReturnLen > sizeof(SerialBuffer))
				{
					SerialReturnLen = sizeof(SerialBuffer);
				}

				for(loop = 0; loop < SerialReturnLen; loop++)
				{
					printf("%02X ",SerialBuffer[loop]);
				}
				printf("\n\n");
			}
		}
		else
		{
			RockForceGetErrorString(Status,ErrBuf,sizeof(ErrBuf));
			printf("Error: %s\n",ErrBuf);

			if(SerialReturnLen > sizeof(SerialBuffer))
			{
				SerialReturnLen = sizeof(SerialBuffer);
			}

			printf("Buffer = ");
			for(loop = 0; loop < SerialReturnLen; loop++)
			{
				printf("%02X ",SerialBuffer[loop]);
			}

			printf("\n\n");
			close(hIO);
			exit(1);
		}
	}

	/*
	** Process commands...
	*/

	if(!pCommand)
	{
		close(hIO);
		exit(0);
	}

	if(strcmp(pCommand,"reset") == 0)
	{
		_u32	Status;

		printf("Resetting RockForce port %d on board @ 0x%04X...", (u_int) SBIndex+1, (u_int) SBDevice);

		Status = RockForcePortReset(SBDevice,SBIndex,RESET_OPENCHECK);

		if(Status == RF_SUCCESS)
		{
			printf("Done!\n");
		}
		else
		{
			RockForceGetErrorString(Status,ErrBuf,sizeof(ErrBuf));
			printf("Error: %s\n",ErrBuf);
		}

		close(hIO);
		exit(0);
	}

	if(strcmp(pCommand,"dumpuart") == 0)
	{
		_u8	Reg = 0;

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_RBR,&Reg) == RF_SUCCESS)
			printf("RBR = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_IER,&Reg) == RF_SUCCESS)
			printf("IER = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_ISR,&Reg) == RF_SUCCESS)
			printf("ISR = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_LCR,&Reg) == RF_SUCCESS)
			printf("LCR = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_MCR,&Reg) == RF_SUCCESS)
			printf("MCR = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_LSR,&Reg) == RF_SUCCESS)
			printf("LSR = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_MSR,&Reg) == RF_SUCCESS)
			printf("MSR = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_SCR,&Reg) == RF_SUCCESS)
			printf("SCR = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_DLL,&Reg) == RF_SUCCESS)
			printf("DLL = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,UART_DLM,&Reg) == RF_SUCCESS)
			printf("DLM = %02X\n",Reg);

		if(RockForcePortReadUARTReg(SBDevice,SBIndex,0x14,&Reg) == RF_SUCCESS)
			printf("MCX = %02X\n",Reg);

		close(hIO);
		exit(0);
	}

	if(strcmp(pCommand,"wedge") == 0)
	{
		_u8	DllValue;
		_u8	DlmValue;

		printf("Force wedging RockForce port %d on board @ 0x%04X\n", (int) SBIndex+1, (u_int) SBDevice);

		RockForcePortReadUARTReg(SBDevice,SBIndex,UART_DLL,&DllValue);
		RockForcePortReadUARTReg(SBDevice,SBIndex,UART_DLM,&DlmValue);

		RockForcePortWriteUARTReg(SBDevice,SBIndex,UART_DLM,(_u8)(0xF7));
		RockForcePortWriteUARTReg(SBDevice,SBIndex,UART_DLL,DllValue);

		RockForcePortWriteUARTReg(SBDevice,SBIndex,UART_DLM,DlmValue);
		RockForcePortWriteUARTReg(SBDevice,SBIndex,UART_DLL,DllValue);

		close(hIO);
		exit(0);
	}

	if(pCommand)
	{
		if(!strcmp(pCommand,"serial") == 0)
		{
			fprintf(stderr,"ERROR: Unknown command \"%s\".\n",pCommand);
			close(hIO);
			exit(1);
		}
	}
	close(hIO);
	exit(0);

} /* main */

/*****************************************************************************
************************                               ***********************
************************   RockForceReadSerialNumber   ***********************
************************                               ***********************
******************************************************************************

prototype:	_u32	RockForceReadSerialNumber(_u32 hDevice,pu8 pBuffer,_u32 BufLen,pu32 pBytesRead)

description:	Read serial number from 1-wire interface in sideband memory space

parameters:	hDevice is the device to access
		pBuffer points to the buffer to return serial number string
		BufLen is the length of the buffer
		pBytesRead points to a variable to return the number of bytes read

returns:	RF_SUCCESS
		RF_INVALID_PARAMETER

*/

_u32	RockForceReadSerialNumber(_u32 hDevice,pu8 pBuffer,_u32 BufLen,pu32 pBytesRead)
{
	char	SerialBuffer[] = "ABCDEFGH";		/* Buffer to store serial string */
	_u32	SerialBufLen = sizeof(SerialBuffer)-1;
	_u32	loop;
	_u32	retries;
	_u8	Crc8;

	/*
	** Entry checks...
	*/

	if((pBuffer == NULL)||(BufLen == 0))
	{
		return(RF_INVALID_BUFFER);
	}

	/*
	** Read the serial number from the 1-Wire bus...
	*/

	WriteReg(SB_1WIRE_CMD,SB_1WIRE_RESET);

	retries = 10;
	while(!(ReadReg(SB_1WIRE_STAT)&SB_1WIRE_PD))
	{
		if(!retries--)
		{
			break;
		}
	}

	Write1WireBus(hDevice,SB_1WIRE_READ_ROM);

	if(Read1WireBus(hDevice) == SB_1WIRE_READ_ROM)
	{
		for(loop = 0; loop < SerialBufLen; loop++)
		{
			/*
			** Write a read command (0xFF) to the bus to prompt a read...
			*/

			Write1WireBus(hDevice,SB_1WIRE_READ);
			SerialBuffer[loop] = Read1WireBus(hDevice);
		}
	}

	/*
	** Transfer the serial number...
	*/

	if(BufLen >= SerialBufLen)
	{
		/*
		** Check 8-CRC checksum...
		*/

		Crc8 = 0;
		for(loop = 0; loop < SerialBufLen; loop++)
		{
			Crc8 = Fcs8(Crc8,SerialBuffer[loop]);
		}

		if(Crc8 == 0)
		{
			_u32	TransferSize = SerialBufLen-2;

			/*
			** If CRC is good, just return the serial number
			*/

			for(loop = 0; loop < TransferSize; loop++)
			{
				pBuffer[TransferSize-loop-1] = SerialBuffer[loop+1];
			}

			if(pBytesRead)
			{
				*pBytesRead = SerialBufLen-2;
			}

			return(RF_SUCCESS);
		}

		/*
		** If CRC failed, return the whole buffer for debug...
		*/
		
		for(loop = 0; loop < SerialBufLen; loop++)
		{
			pBuffer[SerialBufLen-loop-1] = SerialBuffer[loop+1];
		}

		if(pBytesRead)
		{
			*pBytesRead = SerialBufLen;
		}

		return(RF_SERIAL_NUMBER_BAD_CRC);
	}

	return(RF_BUFFER_TOO_SMALL);

} /* RockForceReadSerialNumber */

_u8	Read1WireBus(_u32 hDevice)
{
	int		retries = 10;
	_u8		stat = 0;
	_u8		value = 0;

	/*
	** Wait for the receive buffer full flag...
	*/

	while(!((stat = ReadReg(SB_1WIRE_STAT)) & SB_1WIRE_RBF))
	{
		if(!retries--)
		{
			return('*');
		}
	}

	/*
	** Read from the 1-wire bus...
	*/

	value = ReadReg(SB_1WIRE_RX);
	return(value);

} /* Read1WireBus */

void	Write1WireBus(_u32 hDevice,_u8 Value)
{
	int		retries = 10;
	_u8		stat;

	/*
	** Wait for transmit buffer empty
	*/

	while(!((stat = ReadReg(SB_1WIRE_STAT)) & SB_1WIRE_TBE))
	{
		if(!retries--)
		{
			return;
		}
	}

	/*
	** Write to the 1-wire bus...
	*/

	WriteReg(SB_1WIRE_TX,Value);
	
} /* Write1WireBus */

_u8	_uinb(_u32 port)
{
	_u8	retval = 0;
	_u32	retries = 100;

	if(hIO)
	{
		do {
			SysTimeDelay(10);		/* Wait 10mS */
			lseek(hIO,port,0);
			read(hIO,&retval,1);
		} while(!retval && --retries);
	}

	return(retval);

} /* _uinb */

void	_uoutb(_u32 port,_u8 val)
{
	SysTimeDelay(10);		/* Wait 10mS */
	if(hIO)
	{
		lseek(hIO,port,0);
		write(hIO,&val,1);
	}

} /* _uoutb */

static _u8 Fcs8Table[] = 
{
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53
};

_u8	Fcs8(_u8 Crc8,_u8 x)
{
   Crc8 = Fcs8Table[Crc8 ^ x];
   return(Crc8);

} /* Fcs8 */

/*****************************************************************************
*************************                              ***********************
*************************   RockForceGetErrorString    ***********************
*************************                              ***********************
******************************************************************************

prototype:	_u32	RockForceGetErrorString(_u8 Error,pu8 pBuffer,_u32 BufLen)

description:	Return an error code string for the specified RockForce error

parameters:	Error is the error code to identify
		pBuffer points to the buffer to return serial number string
		BufLen is the length of the buffer

returns:	RF_SUCCESS
		
*/

_u32	RockForceGetErrorString(_u32 Error,pu8 pBuffer,_u32 BufLen)
{
	char	ErrBuf[100];

	switch(Error)
	{
	case	RF_SUCCESS:
		sprintf(ErrBuf,"(%ld) SUCCESS",Error);
		break;

	case	RF_INVALID_PARAMETER:
		sprintf(ErrBuf,"(%ld) Invalid Parameter",Error);
		break;

	case	RF_INVALID_PORT:
		sprintf(ErrBuf,"(%ld) Invalid Port",Error);
		break;

	case	RF_INVALID_BUFFER:
		sprintf(ErrBuf,"(%ld) Invalid Buffer",Error);
		break;

	case	RF_INVALID_REGISTER:
		sprintf(ErrBuf,"(%ld) Invalid Register",Error);
		break;

	case	RF_INVALID_ADDRESS:
		sprintf(ErrBuf,"(%ld) Invalid Address",Error);
		break;

	case	RF_INVALID_MODE:
		sprintf(ErrBuf,"(%ld) Invalid Mode",Error);
		break;

	case	RF_BUFFER_TOO_SMALL:
		sprintf(ErrBuf,"(%ld) Buffer Too Small",Error);
		break;

	case	RF_SERIAL_NUMBER_BAD_CRC:
		sprintf(ErrBuf,"(%ld) Serial Number Bad CRC",Error);
		break;

	default:
		sprintf(ErrBuf,"(%ld) Unknown Error Code",Error);
		break;
	}

	/*
	** Check user buffer...
	*/

	if(BufLen < strlen(ErrBuf))
	{
		return(RF_BUFFER_TOO_SMALL);
	}

	if(pBuffer == NULL)
	{
		return(RF_INVALID_BUFFER);
	}

	sprintf(pBuffer,"%s",ErrBuf);

	return(RF_SUCCESS);

} /* RockForceGetErrorString */

/*****************************************************************************
************************                              ************************
************************   RockForcePortReadUARTReg   ************************
************************                              ************************
******************************************************************************

prototype:	_u32	RockForcePortReadUARTReg(_u32 hDevice,_u32 Port,_u32 Reg,pu8 pValue)

description:	Read specified port's UART register from the sideband memory space

parameters:	hDevice is the device to access
		Port specifies the port to access
		Reg is the UART register to access
		pValue points to space to store result

returns:	RF_SUCCESS
		RF_INVALID_PARAMETER

*/

_u32	RockForcePortReadUARTReg(_u32 hDevice,_u32 Port,_u32 Reg,pu8 pValue)
{
	_u8	Val = 0;

	/*
	** Entry checks...
	*/

	if(Port >= RF_MAX_PORT)
	{
		return(RF_INVALID_PORT);
	}

	/*
	** Read the specified UART register...
	*/

	switch(Reg)
	{
	case	UART_RBR:
	case	UART_IER:
	case	UART_ISR:
	case	UART_LCR:
	case	UART_MCR:
	case	UART_LSR:
	case	UART_MSR:
	case	UART_SCR:
	{
		_u8	LCRVal;

		LCRVal = ReadUARTReg(UART_LCR);
		WriteUARTReg(UART_LCR,(_u8)(LCRVal&~LCR_DLAB));
		Val = ReadUARTReg(Reg);
		WriteUARTReg(UART_LCR,LCRVal);
		break;
	}

	case	UART_DLL:
	case	UART_DLM:
	case	UART_MCX|0x10:
	{
		_u8	LCRVal;

		LCRVal = ReadUARTReg(UART_LCR);
		WriteUARTReg(UART_LCR,(_u8)(LCRVal|LCR_DLAB));
		Val = ReadUARTReg(Reg&UART_REG_MASK);
		WriteUARTReg(UART_LCR,LCRVal);
		break;
	}
	default:
		return(RF_INVALID_REGISTER);
	}

	/*
	** Store it
	*/

	if(pValue)
	{
		*pValue = Val;
	}

	return(RF_SUCCESS);

} /* RockForcePortReadUARTReg */

_u8	ReadUARTRegister(_u32 hDevice,_u32 Port,_u32 Reg)
{
	_u32	UartStride = SB_UART_STRIDE;
	_u32	UartOffset = SB_UART_ADDR + Port * SB_UART_SPAN * UartStride;
	_u8	Val = 0;

	Val = ReadReg(UartOffset + Reg * UartStride);

	return(Val);
}

/*****************************************************************************
************************                               ***********************
************************   RockForcePortWriteUARTReg   ***********************
************************                               ***********************
******************************************************************************

prototype:	_u32	RockForcePortWriteUARTReg(_u32 hDevice,_u32 Port,_u32 Reg,_u8 Value)

description:	Read specified port's UART register from the sideband memory space

parameters:	hDevice is the device to access
		Port specifies the port to access
		Reg is the UART register to access
		Value is the value to store

returns:	RF_SUCCESS
		RF_INVALID_PARAMETER

*/

_u32	RockForcePortWriteUARTReg(_u32 hDevice,_u32 Port,_u32 Reg,_u8 Value)
{
	/*
	** Entry checks...
	*/

	if(Port >= RF_MAX_PORT)
	{
		return(RF_INVALID_PORT);
	}

	/*
	** Write the specified UART register...
	*/

	switch(Reg)
	{
	case	UART_RBR:
	case	UART_IER:
	case	UART_ISR:
	case	UART_LCR:
	case	UART_MCR:
	case	UART_LSR:
	case	UART_MSR:
	case	UART_SCR:
	{
		_u8	LCRVal;

		LCRVal = ReadUARTReg(UART_LCR);
		WriteUARTReg(UART_LCR,(_u8)(LCRVal&~LCR_DLAB));
		WriteUARTReg(Reg,Value);
		WriteUARTReg(UART_LCR,LCRVal);

		break;
	}

	case	UART_DLL:
	case	UART_DLM:
	case	UART_MCX|0x10:
	{
		_u8	LCRVal;

		LCRVal = ReadUARTReg(UART_LCR);
		WriteUARTReg(UART_LCR,(_u8)(LCRVal|LCR_DLAB));
		WriteUARTReg(Reg&UART_REG_MASK,Value);
		WriteUARTReg(UART_LCR,LCRVal);

		break;
	}
	default:
		return(RF_INVALID_REGISTER);
	}

	return(RF_SUCCESS);

} /* RockForcePortWriteUARTReg */

void	WriteUARTRegister(_u32 hDevice,_u32 Port,_u32 Reg,_u8 Val)
{
	_u32	UartStride = SB_UART_STRIDE;
	_u32	UartOffset = SB_UART_ADDR + Port * SB_UART_SPAN * UartStride;

	WriteReg(UartOffset + Reg * UartStride,Val);
}

/*****************************************************************************
***************************                        ***************************
***************************   RockForcePortReset   ***************************
***************************                        ***************************
******************************************************************************

prototype:	_u32	RockForcePortReset(_u32 hDevice,_u32 Port,_u32 Mode)

description:	Reset the specified port

parameters:	hDevice is the device to access
		Port is the port number to access (0..7)
		Mode specifies type of reset:
			RESET_ONLY
			RESET_OPENCHECK
			RESET_OPENFORCE

returns:	RF_SUCCESS
		RF_INVALID_PARAMETER

*/

_u32	RockForcePortReset(_u32 hDevice,_u32 Port,_u32 Mode)
{
	_u32	retries;
	_u8	RegVal;

	/*
	** Entry checks...
	*/

	if(Port >= RF_MAX_PORT)
	{
		return(RF_INVALID_PORT);
	}

	/*
	** Reset the port...
	*/

	switch(Mode)
	{
	case	RESET_ONLY:

		/*
		** Just reset the port...
		*/

		RegVal = ReadReg(SB_RESET_ADDR);
		WriteReg(SB_RESET_ADDR,(_u8)(0x01 << Port));
		WriteReg(SB_RESET_ADDR,RegVal);

		/*
		** Wait for the modem to boot and the registers to settle...
		*/

		retries = 200;
		while(retries--)
		{
			if(ReadUARTReg(UART_IER) != 0xFF)
			{
				break;
			}
		}
		break;

	case	RESET_OPENCHECK:
	{
		_u8	IERVal;
		_u8	LCRVal;
		_u8	MCRVal;
		_u8	DLLVal;
		_u8	DLMVal;

		/*
		** If port looks open, then...
		** read UARTs registers before the reset and then restore afterwards...
		*/

		LCRVal = ReadUARTReg(UART_LCR);
		WriteUARTReg(UART_LCR,(_u8)(LCRVal&~LCR_DLAB));
		IERVal = ReadUARTReg(UART_IER);
		MCRVal = ReadUARTReg(UART_MCR);
		WriteUARTReg(UART_LCR,(_u8)(LCRVal|LCR_DLAB));
		DLLVal = ReadUARTReg(UART_DLL&UART_REG_MASK);
		DLMVal = ReadUARTReg(UART_DLM&UART_REG_MASK);

		RegVal = ReadReg(SB_RESET_ADDR);
		WriteReg(SB_RESET_ADDR,(_u8)(0x01 << Port));
		WriteReg(SB_RESET_ADDR,RegVal);

		/*
		** Wait for the modem to boot and the registers to settle...
		*/

		retries = 200;
		while(retries--)
		{
			if((ReadUARTReg(UART_IER) != 0xFF)&&(ReadUARTReg(UART_IER) != 0x7F))
			{
				break;
			}
		}

		WriteUARTReg(UART_LCR,(_u8)(LCRVal|LCR_DLAB));
		WriteUARTReg(UART_DLL&UART_REG_MASK,DLLVal);
		WriteUARTReg(UART_DLM&UART_REG_MASK,DLMVal);
		WriteUARTReg(UART_LCR,(_u8)(LCRVal&~LCR_DLAB));
		WriteUARTReg(UART_FCR,0x01);		/* Enable FIFOs */
		WriteUARTReg(UART_MCR,MCRVal);
		WriteUARTReg(UART_IER,IERVal);
		WriteUARTReg(UART_LCR,LCRVal);
		break;
	}

	default:
		return(RF_INVALID_MODE);
	}

	return(RF_SUCCESS);

} /* RockForcePortReset */

/*****************************************************************************
******************************                  ******************************
******************************   SysTimeStamp   ******************************
******************************                  ******************************
******************************************************************************

prototype:	_u32	SysTimeStamp(void)

description:	Get current time in mS

parameters:	None

returns:	Current timestamp in mS

*/

_u32	LastTime = 0L;

_u32	SysTimeStamp(void)
{
	struct timeb	TimeNow;
	_u32		ThisTime;

	ftime(&TimeNow);				/* Get the current time */

	/*
	** Calculate time in mS...
	*/

	ThisTime = TimeNow.time*1000+TimeNow.millitm;

	/*
	** Check to make sure that a time is not returned which is
	** previous to the last time... (NT does this occasionally)
	*/

	if(ThisTime >= LastTime)
	{
		LastTime = ThisTime;	/* OK */
	}
	else
	{
		ThisTime = LastTime;	/* Not OK - return the LastTime again... */
	}

	return(ThisTime);		/* Return as mS */

} /* SysTimeStamp */

/*****************************************************************************
*****************************                    *****************************
*****************************   SysTimeElapsed   *****************************
*****************************                    *****************************
******************************************************************************

prototype:	_u32	SysTimeElapsed(_u32 TimeStampmS)

description:	Calculate the number of mS elapsed since the timestamp

parameters:	TimeStampmS is a previously obtained timestamp

returns:	Elapsed mS since TimeStampmS

*/

_u32	SysTimeElapsed(_u32 TimeStampmS)
{
	return(SysTimeStamp() - TimeStampmS);

} /* SysTimeElapsed */

/*****************************************************************************
******************************                  ******************************
******************************   SysTimeDelay   ******************************
******************************                  ******************************
******************************************************************************

prototype:	_u32	SysTimeDelay(_u32 DelaymS)

description:	Delay for the specified number of mS

parameters:	DelaymS is the number of mS to wait

returns:	Actual number of mS delayed for

*/

_u32	SysTimeDelay(_u32 DelaymS)
{
	_u32	TimeStamp = SysTimeStamp();

	while(SysTimeElapsed(TimeStamp) < DelaymS)
	{
	}

	return(SysTimeElapsed(TimeStamp));

} /* SysTimeDelay */

/* End of ROCKFORCE.C */
